#!/usr/bin/env python
#
# Authors:
# rafael@postgresql.org.es / http://www.postgresql.org.es/
#
# Copyright (c) 2016 USIT-University of Oslo
#
# zabbix_elasticsearch_node_stats.py: Used by zabbix_agent to pull
# elasticsearch cluster health information from an ES cluster and send
# this information to Zabbix via trappers.
#
# zabbix_elasticsearch_node_stats.py is free software: you can
# redistribute it and/or modify it under the terms of the GNU General
# Public License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# zabbix_elasticsearch_node_stats.py is distributed in the hope
# that it will be useful, but WITHOUT ANY WARRANTY; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with sms_send.  If not, see <http://www.gnu.org/licenses/>.
#

#
# This script gets ES node stats information from the ES rest API,
# extracts the parameters we have defined in stats_keys{} and sends
# data back to zabbix via zabbix_sender to defined trap-items.
#
# The script is executed via zabbix_agent and is defined in an
# UserParameter that will return 0 (execution OK) or 1 (execution
# ERROR) so zabbix can register if it cannot get data from ES.
#
# UserParameter=get.es.node.stats[*],/usr/bin/zabbix_elasticsearch_node_stats.py $1
#

import requests
import json
import sys
import os
import time
import tempfile

# Elasticsearch clients
elasticsearch_clients = ['es-client01.uio.no','es-client02.uio.no']

# Zabbix proxy
zabbix_proxy = ['zabbix-proxy-prod01.uio.no','zabbix-proxy-prod02.uio.no']

# Path to zabbix_sender
zabbix_sender = '/usr/bin/zabbix_sender'

# Temp file with full json output
tmp_stat_file = tempfile.NamedTemporaryFile(delete=False,dir='/tmp')

# Item prefix
item_prefix = 'es.node.'

# keys for health page
stats_keys = {  
    "indices.docs.count",
    "indices.docs.deleted",
    "indices.store.size_in_bytes",
    "indices.store.throttle_time_in_millis",
    "indices.indexing.index_total",
    "indices.indexing.index_time_in_millis",
    "indices.indexing.index_current",
    "indices.indexing.index_failed",
    "indices.indexing.delete_total",
    "indices.indexing.delete_time_in_millis",
    "indices.indexing.delete_current",
    "indices.get.total",
    "indices.get.time_in_millis",
    "indices.get.exists_total",
    "indices.get.exists_time_in_millis",
    "indices.get.missing_total",
    "indices.get.missing_time_in_millis",
    "indices.get.current",
    "indices.search.open_contexts",
    "indices.search.query_total",
    "indices.search.query_time_in_millis",
    "indices.search.query_current",
    "indices.search.fetch_total",
    "indices.search.fetch_time_in_millis",
    "indices.search.fetch_current",
    "indices.merges.current",
    "indices.merges.current_docs",
    "indices.merges.total_size_in_bytes",
    "indices.merges.total",
    "indices.merges.total_time_in_millis",
    "indices.merges.total_docs",
    "indices.merges.total_size_in_bytes",
    "indices.merges.total_stopped_time_in_millis",
    "indices.fielddata.memory_size_in_bytes",
    "indices.fielddata.evictions",
    "indices.segments.count",
    "process.open_file_descriptors",
    "process.max_file_descriptors",
    "jvm.mem.heap_used_in_bytes",
    "jvm.mem.heap_used_percent",
    "jvm.mem.heap_committed_in_bytes",
    "jvm.mem.heap_max_in_bytes",
    "jvm.mem.non_heap_used_in_bytes",
    "jvm.mem.non_heap_committed_in_bytes",
    "jvm.threads.count",
    "jvm.threads.peak_count",
    "jvm.gc.collectors.young.collection_count",
    "jvm.gc.collectors.young.collection_time_in_millis",
    "jvm.gc.collectors.old.collection_count",
    "jvm.gc.collectors.old.collection_time_in_millis",
    "transport.server_open",
    "transport.rx_count",
    "transport.rx_size_in_bytes",
    "transport.tx_count",
    "transport.tx_size_in_bytes"
    }

# ############################################
# getKeys()
# ############################################

def getKeys(json_data,keys,node_fqdn):
    result=''
    
    for key in keys:
        attributes=key.split('.')
        value=json_data

        for index in range(len(attributes)):
            value=value.get(attributes.pop(0),{})

            if value=={}:
                continue

        result += node_fqdn + ' ' + item_prefix + "{0} {1}\n".format(key,value)

    return result


# ############################################
# get_node_stats_data()
# ############################################

def get_node_stats_data(node_fqdn):
    """
    Get ES node stats data
    """

    #
    # We try all ES clients defined in elasticsearch_clients[] until
    # one of them returns an ansver with a 200 status code.
    #

    for client in elasticsearch_clients:

        try:
            request_data = requests.get("http://" + client + ":9200/_nodes/" + node_fqdn + "/stats?all")

            if request_data.status_code != 200:
                continue
            
            stats_data = request_data.json()
            
            break

        except Exception:
            pass
           
    try:

        for node_id in stats_data['nodes']:
            result = getKeys(stats_data['nodes'][node_id],stats_keys,node_fqdn)
            
    except Exception, e:
        print "1"
        sys.exit(1)

    return result

# ############################################
# Main
# ############################################

if __name__ == '__main__':

    try:

        if len(sys.argv) == 2:

            node_fqdn = sys.argv[1].lower()
            
            result = get_node_stats_data(node_fqdn)

            #
            # We create a file with the data that zabbix_sender will
            # send in a bulk execution.
            #

            with open(tmp_stat_file.name,'w') as f: 
                f.write(result)
               
            #
            # The monitoring of this host can be done by any of the
            # zabbix proxyer defined in zabbix_proxy[]. We try all of
            # them until one of them accepts our data
            #
    
            for proxy in zabbix_proxy:
                returncode = os.system(zabbix_sender + ' -z ' + proxy + ' -i ' + tmp_stat_file.name + ' > /dev/null 2>&1')

                if returncode == 0:
                    break

            if returncode != 0:
                print "1"
                sys.exit(1)
            
        else:
            print "1"
            sys.exit(1)

    except Exception, e:
        print "1"
        sys.exit(1)

    # Delete temp file with zabbix_sender data
    os.remove(tmp_stat_file.name)

    # Return value 0 = execution OK
    print "0"
