#!/usr/bin/env python
#
# Authors:
# rafael@postgresql.org.es / http://www.postgresql.org.es/
#
# Copyright (c) 2015 USIT-University of Oslo
#
# zabbix_filesystem_limits.py: Used by zabbix_agent to find out if
# custom filesystem limits have been defined for some of the mounted
# filesystem in a server.
#
# zabbix_filesystem_limits.py is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either version
# 3 of the License, or (at your option) any later version.
#
# zabbix_filesystem_limits.py is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with sms_send.  If not, see <http://www.gnu.org/licenses/>.
#

import os
import sys
import glob

# ###################################################################
# Method: get_local_filesystem_limits()
# Parameters: filesystem, limit_type
#
# Used to read the file /etc/zabbix_filesystems_limits.conf if it
# exists and the custom filesystems limits defined in this file.
#
# It will return -1 if no custom limit has been defined, or the filesystem
# is not defined in the file.
#
# Limit_type valid values: [percent_limit | size_limit]
#
# The format used to define custom limits for a filesystem is:
#
# <filesystem>::<percent_limit>::<size_limit>
#
# Filesystems can match a specified pattern according to the rules
# used by the Unix shell. No tilde expansion is done, but *, ?, and
# character ranges expressed with [] will be correctly matched. For a
# literal match, wrap the meta-characters in brackets. For example,
# '[?]' matches the character '?'.
#
# Only mounted filesystem will be considered when reading filesystems
# limits definitions in /etc/zabbix_filesystems_limits.conf.
#
# Examples:
#
# Filesystem /mnt/my_filesystem limits will be 98% and 2048M:
# /mnt/my_filesystem::98::2048M
#
# Filesystems starting with /mnt limits will be 98% and 2048M:
# /mnt/*::98::2048M
#
# Filesystems starting with /mnt and a or b in the path third level
# limits will be 98% and 2048M: 
# /mnt/*/[ab]*::98::2048M
#
# ###################################################################


def get_local_filesystem_limits(filesystem, limit_type):

    defined_filesystems = dict()
    config_file = '/etc/zabbix_filesystems_limits.conf'

    try:

        if os.path.isfile(config_file):
            fd = open(config_file, "r")

            for line in fd:
                line = line.replace('\n', '').strip()

                #
                # Lines starting with # or empty lines are ignored.
                #

                if line.find('#', 0) == -1 and line.strip() != '':
                    filesystem_limit_definition = line.split('::')

                    #
                    # Only lines with the three parameters defined
                    # will be considered.
                    #

                    if len(filesystem_limit_definition) == 3:

                        defined_fs = filesystem_limit_definition[0]

                        if filesystem_limit_definition[1] == '':
                            percent_limit = "-1"
                        else:
                            percent_limit = filesystem_limit_definition[1]

                        if filesystem_limit_definition[2] == '':
                            size_limit = "-1"
                        else:
                            if filesystem_limit_definition[2][-1:].upper() == 'M':
                                size_limit = float(filesystem_limit_definition[2][:-1])*1024*1024

                            elif filesystem_limit_definition[2][-1:].upper() == 'G':
                                size_limit = float(filesystem_limit_definition[2][:-1])*1024*1024*1024

                            elif filesystem_limit_definition[2][-1:].upper() == 'T':
                                size_limit = float(filesystem_limit_definition[2][:-1])*1024*1024*1024*1024

                            else:
                                size_limit = "-1"

                        # Normalized absolutized version of the pathname
                        # if filesystem does not include an absolute path

                        if not os.path.isabs(defined_fs):
                            defined_fs = os.path.abspath(defined_fs)

                        #
                        # Finds all the pathnames matching a specified
                        # pattern according to the rules used by the Unix
                        # shell. No tilde expansion is done, but *, ?, and
                        # character ranges expressed with [] will be
                        # correctly matched. For a literal match, wrap the
                        # meta-characters in brackets. For example, '[?]'
                        # matches the character '?'.
                        #

                        expanded_paths = glob.glob(defined_fs)

                        #
                        # Only mounted filesystem will be considered when
                        # reading filesystems limits definitions in
                        # /etc/zabbix_filesystems_limits.conf.
                        #

                        for expanded_fs in expanded_paths:
                            if os.path.ismount(expanded_fs):
                                defined_filesystems[expanded_fs] = [percent_limit, size_limit]

            #
            # Find out if the filesystem we are asking for exists in
            # /etc/zabbix_filesystems_limits.conf. If it does not exists,
            # return -1
            #

            if defined_filesystems.has_key(filesystem):

                if limit_type == 'percent_limit':
                    print defined_filesystems[filesystem][0]

                elif limit_type == 'size_limit':
                    print defined_filesystems[filesystem][1]

            else:
                print "-1"

        #
        # Return -1 if the file /etc/filesystems_limits.conf does not
        # exist.
        #

        else:
            print "-1"

    except Exception, exc:
        raise exc


# ############################################
# Main
# ############################################

if __name__ == '__main__':

    try:

        if len(sys.argv) == 3:
            fs = sys.argv[1]
            limit_t = sys.argv[2]

            get_local_filesystem_limits(fs, limit_t)

        else:
            print "Error: Wrong number of parameters"
            print 'Format: ' + sys.argv[0] + ' <filesystem> <percent_limit | size_limit>'
            sys.exit(1)

    except Exception, e:
        print e
        sys.exit(1)
